@@ -1,19 +1,21 @@
 {-# LANGUAGE OverloadedStrings #-}
 
 module Language.Haskell.GhcMod.CabalApi (
-    fromCabalFile
+    getCompilerOptions
   , parseCabalFile
   , cabalAllBuildInfo
   , cabalDependPackages
   , cabalSourceDirs
-  , getGHCVersion
+  , cabalAllTargets
   ) where
 
 import Control.Applicative ((<$>))
 import Control.Exception (throwIO)
-import Data.List (intercalate)
+import Control.Monad (filterM)
+import CoreMonad (liftIO)
 import Data.Maybe (maybeToList)
 import Data.Set (fromList, toList)
+import Distribution.ModuleName (ModuleName,toFilePath)
 import Distribution.Package (Dependency(Dependency)
                            , PackageName(PackageName)
                            , PackageIdentifier(pkgName))
@@ -26,34 +28,23 @@ import Distribution.Simple.Program.Types (programName, programFindVersion)
 import Distribution.System (buildPlatform)
 import Distribution.Text (display)
 import Distribution.Verbosity (silent)
-import Distribution.Version (versionBranch, Version)
+import Distribution.Version (Version)
 import Language.Haskell.GhcMod.Types
 import System.Directory (doesFileExist)
 import System.FilePath
 
 ----------------------------------------------------------------
 
--- | Parsing a cabal file in 'Cradle' and returns
---   options for GHC, include directories for modules and
---   package names of dependency.
-fromCabalFile :: [GHCOption]
-              -> Cradle
-              -> IO ([GHCOption],[IncludeDir],[Package])
-fromCabalFile ghcOptions cradle =
-    parseCabalFile cfile >>= cookInfo ghcOptions cradle
-  where
-    Just cfile = cradleCabalFile cradle
-
-cookInfo :: [GHCOption] -> Cradle -> PackageDescription
-         -> IO ([GHCOption],[IncludeDir],[Package])
-cookInfo ghcOptions cradle cabal = do
-    gopts <- getGHCOptions ghcOptions cdir $ head buildInfos
-    return (gopts,idirs,depPkgs)
+-- | Getting necessary 'CompilerOptions' from three information sources.
+getCompilerOptions :: [GHCOption] -> Cradle -> PackageDescription -> IO CompilerOptions
+getCompilerOptions ghcopts cradle pkgDesc = do
+    gopts <- getGHCOptions ghcopts cradle cdir $ head buildInfos
+    return $ CompilerOptions gopts idirs depPkgs
   where
     wdir       = cradleCurrentDir cradle
     Just cdir  = cradleCabalDir   cradle
     Just cfile = cradleCabalFile  cradle
-    buildInfos = cabalAllBuildInfo cabal
+    buildInfos = cabalAllBuildInfo pkgDesc
     idirs      = includeDirectories cdir wdir $ cabalSourceDirs buildInfos
     depPkgs    = removeThem problematicPackages $ removeMe cfile $ cabalDependPackages buildInfos
 
@@ -77,12 +68,14 @@ problematicPackages = [
 -- Include directories for modules
 
 cabalBuildDirs :: [FilePath]
-cabalBuildDirs = ["dist/build"]
+cabalBuildDirs = ["dist/build", "dist/build/autogen"]
 
 includeDirectories :: FilePath -> FilePath -> [FilePath] -> [FilePath]
 includeDirectories cdir wdir dirs = uniqueAndSort (extdirs ++ [cdir,wdir])
   where
-    extdirs = map (cdir </>) $ dirs ++ cabalBuildDirs
+    extdirs = map expand $ dirs ++ cabalBuildDirs
+    expand "."    = cdir
+    expand subdir = cdir </> subdir
 
 ----------------------------------------------------------------
 
@@ -105,12 +98,13 @@ parseCabalFile file = do
 
 ----------------------------------------------------------------
 
-getGHCOptions :: [GHCOption] -> FilePath -> BuildInfo -> IO [GHCOption]
-getGHCOptions ghcOptions cdir binfo = do
+getGHCOptions :: [GHCOption] -> Cradle -> FilePath -> BuildInfo -> IO [GHCOption]
+getGHCOptions ghcopts cradle cdir binfo = do
     cabalCpp <- cabalCppOptions cdir
     let cpps = map ("-optP" ++) $ cppOptions binfo ++ cabalCpp
-    return $ ghcOptions ++ exts ++ [lang] ++ libs ++ libDirs ++ cpps
+    return $ ghcopts ++ pkgDb ++ exts ++ [lang] ++ libs ++ libDirs ++ cpps
   where
+    pkgDb = cradlePackageDbOpts cradle
     lang = maybe "-XHaskell98" (("-X" ++) . display) $ defaultLanguage binfo
     libDirs = map ("-L" ++) $ extraLibDirs binfo
     exts = map (("-X" ++) . display) $ usedExtensions binfo
@@ -159,18 +153,6 @@ uniqueAndSort = toList . fromList
 
 ----------------------------------------------------------------
 
--- | Getting GHC version. 7.6.3 becames 706 in the second of the result.
-getGHCVersion :: IO (GHCVersion, Int)
-getGHCVersion = toTupple <$> getGHC
-  where
-    toTupple v
-      | length vs < 2 = (verstr, 0)
-      | otherwise     = (verstr, ver)
-      where
-        vs = versionBranch v
-        ver = (vs !! 0) * 100 + (vs !! 1)
-        verstr = intercalate "." . map show $ vs
-
 getGHCId :: IO CompilerId
 getGHCId = CompilerId GHC <$> getGHC
 
@@ -180,3 +162,41 @@ getGHC = do
     case mv of
         Nothing -> throwIO $ userError "ghc not found"
         Just v  -> return $ v
+
+----------------------------------------------------------------
+
+-- | Extracting all 'Module' 'FilePath's for libraries, executables,
+-- tests and benchmarks.
+cabalAllTargets :: PackageDescription -> IO ([String],[String],[String],[String])
+cabalAllTargets pd = do
+    exeTargets  <- mapM getExecutableTarget $ executables pd
+    testTargets <- mapM getTestTarget $ testSuites pd
+    return (libTargets,concat exeTargets,concat testTargets,benchTargets)
+  where
+    lib = case library pd of
+            Nothing -> []
+            Just l -> libModules l
+
+    libTargets = map toModuleString $ lib
+    benchTargets = map toModuleString $ concatMap benchmarkModules $ benchmarks  pd
+
+    toModuleString :: ModuleName -> String
+    toModuleString mn = fromFilePath $ toFilePath mn
+
+    fromFilePath :: FilePath -> String
+    fromFilePath fp = map (\c -> if c=='/' then '.' else c) fp
+
+    getTestTarget :: TestSuite -> IO [String]
+    getTestTarget ts =
+       case testInterface ts of
+        (TestSuiteExeV10 _ filePath) -> do
+          let maybeTests = [p </> e | p <- hsSourceDirs $ testBuildInfo ts, e <- [filePath]]
+          liftIO $ filterM doesFileExist maybeTests
+        (TestSuiteLibV09 _ moduleName) -> return [toModuleString moduleName]
+        (TestSuiteUnsupported _)       -> return []
+
+    getExecutableTarget :: Executable -> IO [String]
+    getExecutableTarget exe = do
+      let maybeExes = [p </> e | p <- hsSourceDirs $ buildInfo exe, e <- [modulePath exe]]
+      liftIO $ filterM doesFileExist maybeExes
+
