@@ -17,30 +17,27 @@ import Prelude
 -- | Obtaining debug information.
 debugInfo :: Options
           -> Cradle
-          -> GHCVersion
           -> FilePath   -- ^ A target file.
           -> IO String
-debugInfo opt cradle ver fileName = unlines <$> withGHC fileName (debug opt cradle ver fileName)
+debugInfo opt cradle fileName = unlines <$> withGHC fileName (debug opt cradle fileName)
 
 -- | Obtaining debug information.
 debug :: Options
       -> Cradle
-      -> GHCVersion
       -> FilePath     -- ^ A target file.
       -> Ghc [String]
-debug opt cradle ver fileName = do
-    (gopts, incDir, pkgs) <-
+debug opt cradle fileName = do
+    CompilerOptions gopts incDir pkgs <-
         if cabal then
-            liftIO $ fromCabalFile (ghcOpts opt) cradle ||> return (ghcOpts opt, [], [])
+            liftIO (fromCabalFile ||> return simpleCompilerOption)
           else
-            return (ghcOpts opt, [], [])
+            return simpleCompilerOption
     [fast] <- do
         void $ initializeFlagsWithCradle opt cradle gopts True
         setTargetFiles [fileName]
         pure . canCheckFast <$> depanal [] False
     return [
-        "GHC version:         " ++ ver
-      , "Current directory:   " ++ currentDir
+        "Current directory:   " ++ currentDir
       , "Cabal file:          " ++ cabalFile
       , "GHC options:         " ++ unwords gopts
       , "Include directories: " ++ unwords incDir
@@ -49,5 +46,11 @@ debug opt cradle ver fileName = do
       ]
   where
     currentDir = cradleCurrentDir cradle
-    cabal = isJust $ cradleCabalFile cradle
-    cabalFile = fromMaybe "" $ cradleCabalFile cradle
+    mCabalFile = cradleCabalFile cradle
+    cabal = isJust mCabalFile
+    cabalFile = fromMaybe "" mCabalFile
+    origGopts = ghcOpts opt
+    simpleCompilerOption = CompilerOptions origGopts [] []
+    fromCabalFile = parseCabalFile file >>= getCompilerOptions origGopts cradle
+      where
+        file = fromJust mCabalFile
