@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE BangPatterns, CPP #-}
 
 module Language.Haskell.GhcMod.ErrMsg (
     LogReader
@@ -55,7 +55,7 @@ errBagToStrList dflag ls = map (ppErrMsg dflag ls) . reverse . bagToList
 ppErrMsg :: DynFlags -> LineSeparator -> ErrMsg -> String
 ppErrMsg dflag ls err = ppMsg spn SevError dflag ls msg ++ ext
    where
-     spn = head (errMsgSpans err)
+     spn = Gap.errorMsgSpan err
      msg = errMsgShortDoc err
      ext = showMsg dflag ls (errMsgExtraInfo err)
 
