@@ -19,6 +19,9 @@ module Language.Haskell.GhcMod.Gap (
   , infoThing
   , pprInfo
   , HasType(..)
+  , errorMsgSpan
+  , typeForUser
+  , deSugar
 #if __GLASGOW_HASKELL__ >= 702
 #else
   , module Pretty
@@ -30,6 +33,7 @@ import Control.Monad
 import Data.List
 import Data.Maybe
 import Data.Time.Clock
+import Desugar (deSugarExpr)
 import DynFlags
 import ErrUtils
 import FastString
@@ -41,6 +45,8 @@ import Outputable
 import PprTyThing
 import StringBuffer
 import TcType
+import TcRnTypes
+import CoreSyn
 
 import qualified InstEnv
 import qualified Pretty
@@ -207,6 +213,7 @@ setCabalPkg dflag = dopt_set dflag Opt_BuildingCabalPackage
 ----------------------------------------------------------------
 
 addDevPkgs :: DynFlags -> [Package] -> DynFlags
+addDevPkgs df []   = df
 addDevPkgs df pkgs = df''
   where
 #if __GLASGOW_HASKELL__ >= 707
@@ -257,9 +264,9 @@ infoThing str = do
     return $ vcat (intersperse (text "") $ map (pprInfo False) filtered)
 
 #if __GLASGOW_HASKELL__ >= 707
-pprInfo :: PrintExplicitForalls -> (TyThing, GHC.Fixity, [ClsInst], [FamInst]) -> SDoc
-pprInfo pefas (thing, fixity, insts, famInsts)
-    = pprTyThingInContextLoc pefas thing
+pprInfo :: Bool -> (TyThing, GHC.Fixity, [ClsInst], [FamInst]) -> SDoc
+pprInfo _ (thing, fixity, insts, famInsts)
+    = pprTyThingInContextLoc thing
    $$ show_fixity fixity
    $$ InstEnv.pprInstances insts
    $$ pprFamInsts famInsts
@@ -278,3 +285,39 @@ pprInfo pefas (thing, fixity, insts)
       | fx == defaultFixity = Outputable.empty
       | otherwise           = ppr fx <+> ppr (getName thing)
 #endif
+
+----------------------------------------------------------------
+----------------------------------------------------------------
+
+errorMsgSpan :: ErrMsg -> SrcSpan
+#if __GLASGOW_HASKELL__ >= 707
+errorMsgSpan = errMsgSpan
+#else
+errorMsgSpan = head . errMsgSpans
+#endif
+
+typeForUser :: Type -> SDoc
+#if __GLASGOW_HASKELL__ >= 707
+typeForUser = pprTypeForUser
+#else
+typeForUser = pprTypeForUser False
+#endif
+
+deSugar :: TypecheckedModule -> LHsExpr Id -> HscEnv
+         -> IO (Maybe CoreSyn.CoreExpr)
+#if __GLASGOW_HASKELL__ >= 707
+deSugar tcm e hs_env = snd <$> deSugarExpr hs_env modu rn_env ty_env fi_env e
+  where
+    modu = ms_mod $ pm_mod_summary $ tm_parsed_module tcm
+    tcgEnv = fst $ tm_internals_ tcm
+    rn_env = tcg_rdr_env tcgEnv
+    ty_env = tcg_type_env tcgEnv
+    fi_env = tcg_fam_inst_env tcgEnv
+#else
+deSugar tcm e hs_env = snd <$> deSugarExpr hs_env modu rn_env ty_env e
+  where
+    modu = ms_mod $ pm_mod_summary $ tm_parsed_module tcm
+    tcgEnv = fst $ tm_internals_ tcm
+    rn_env = tcg_rdr_env tcgEnv
+    ty_env = tcg_type_env tcgEnv
+#endif
