@@ -1,4 +1,4 @@
-{-# LANGUAGE TupleSections, FlexibleInstances, TypeSynonymInstances #-}
+{-# LANGUAGE TupleSections, FlexibleInstances, TypeSynonymInstances, CPP #-}
 {-# LANGUAGE Rank2Types #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -18,7 +18,6 @@ import Data.List
 import Data.Maybe
 import Data.Ord as O
 import Data.Time.Clock
-import Desugar
 import GHC
 import GHC.SYB.Utils
 import HscTypes
@@ -29,9 +28,7 @@ import qualified Language.Haskell.GhcMod.Gap as Gap
 import Language.Haskell.GhcMod.Gap (HasType(..))
 import Language.Haskell.GhcMod.Types
 import Outputable
-import PprTyThing
 import TcHsSyn (hsPatType)
-import TcRnTypes
 
 ----------------------------------------------------------------
 
@@ -68,12 +65,8 @@ info opt cradle file modstr expr =
 instance HasType (LHsExpr Id) where
     getType tcm e = do
         hs_env <- getSession
-        (_, mbe) <- Gap.liftIO $ deSugarExpr hs_env modu rn_env ty_env e
+        mbe <- Gap.liftIO $ Gap.deSugar tcm e hs_env
         return $ (getLoc e, ) <$> CoreUtils.exprType <$> mbe
-      where
-        modu = ms_mod $ pm_mod_summary $ tm_parsed_module tcm
-        rn_env = tcg_rdr_env $ fst $ tm_internals_ tcm
-        ty_env = tcg_type_env $ fst $ tm_internals_ tcm
 
 instance HasType (LPat Id) where
     getType _ (L spn pat) = return $ Just (spn, hsPatType pat)
@@ -137,7 +130,7 @@ listifyStaged :: Typeable r => Stage -> (r -> Bool) -> GenericQ [r]
 listifyStaged s p = everythingStaged s (++) [] ([] `mkQ` (\x -> [x | p x]))
 
 pretty :: DynFlags -> Type -> String
-pretty dflag = showUnqualifiedOneLine dflag . pprTypeForUser False
+pretty dflag = showUnqualifiedOneLine dflag . Gap.typeForUser
 
 ----------------------------------------------------------------
 
