@@ -17,12 +17,14 @@ data Options = Options {
   , operators     :: Bool
   -- | If 'True', 'browse' also returns types.
   , detailed      :: Bool
+  -- | If 'True', 'browse' will return fully qualified name
+  , qualified     :: Bool
   -- | Whether or not Template Haskell should be expanded.
   , expandSplice  :: Bool
-  -- | The sandbox directory.
-  , sandbox       :: Maybe FilePath
   -- | Line separator string.
   , lineSeparator :: LineSeparator
+  -- | Package id of module
+  , packageId :: Maybe String
   }
 
 -- | A default 'Options'.
@@ -33,9 +35,10 @@ defaultOptions = Options {
   , ghcOpts       = []
   , operators     = False
   , detailed      = False
+  , qualified     = False
   , expandSplice  = False
-  , sandbox       = Nothing
   , lineSeparator = LineSeparator "\0"
+  , packageId     = Nothing
   }
 
 ----------------------------------------------------------------
@@ -85,24 +88,21 @@ data Cradle = Cradle {
   , cradleCabalDir    :: Maybe FilePath
   -- | The file name of the found cabal file.
   , cradleCabalFile   :: Maybe FilePath
-  -- | The sandbox directory. (e.g. \"\/foo\/bar\/packages-\<ver\>.conf/\")
-  , cradlePackageConf :: Maybe FilePath
+  -- | The package db options. ([\"-no-user-package-db\",\"-package-db\",\"\/foo\/bar\/i386-osx-ghc-7.6.3-packages.conf.d\"])
+  , cradlePackageDbOpts :: [GHCOption]
   } deriving (Eq, Show)
 
 ----------------------------------------------------------------
 
--- | A single GHC option, as it would appear on the command line.
+-- | A single GHC command line option.
 type GHCOption  = String
 
--- | Include directories for modules
+-- | An include directory for modules.
 type IncludeDir = FilePath
 
--- | Package names
+-- | A package name.
 type Package    = String
 
--- | GHC version in 'String'.
-type GHCVersion = String
-
 -- | Haskell expression.
 type Expression = String
 
@@ -110,3 +110,10 @@ type Expression = String
 type ModuleString = String
 
 data CheckSpeed = Slow | Fast
+
+-- | Option information for GHC
+data CompilerOptions = CompilerOptions {
+    ghcOptions  :: [GHCOption]  -- ^ Command line options
+  , includeDirs :: [IncludeDir] -- ^ Include directories for modules
+  , depPackages :: [Package]    -- ^ Dependent package names
+  } deriving (Eq, Show)
