@@ -12,7 +12,7 @@
 
 (defun ghc-insert-module ()
   (interactive)
-  (if (not (ghc-which ghc-hoogle-command))
+  (if (not (executable-find ghc-hoogle-command))
       (message "\"%s\" not found" ghc-hoogle-command)
     (let* ((expr0 (ghc-things-at-point))
 	   (expr (ghc-read-expression expr0)))
